/*!
        @file    $Id: test_HotStart_Unitary.cpp #$

        @brief

        @author  Hideo Matsufuru <hideo.matsufuru@kek.jp> (matsufuru)
                 $LastChangedBy: aoym $

        @date    $LastChangedDate: 2013-01-22 13:51:53 #$

        @version $LastChangedRevision: 875 $
*/

#include "parameterManager_YAML.h"
#include "parameters_factory.h"

#include "bridgeIO.h"
using Bridge::vout;

#include "gaugeConfig.h"

#ifdef USE_TEST
#include "test.h"
#endif

#ifdef USE_TESTMANAGER_AUTOREGISTER
#include "testManager.h"
#endif

//====================================================================
//! Test of hot start.

/*!
   [19 Nov 2013 S.Ueda]
 */

namespace Test_HotStart {
  //- test-private parameters
  namespace {
    const std::string filename_input  = "test_HotStart_Unitary.yaml";
    const std::string filename_output = "stdout";

    class Parameters_Test_HotStart : public Parameters {
     public:
      Parameters_Test_HotStart()
      {
        //Register_string("gauge_config_type_input", "NULL");

        Register_string("verbose_level", "NULL");

        Register_double("expected_result", 0.0);
      }
    };
  }

  //- prototype declaration
  int unitary(void);

#ifdef USE_TESTMANAGER_AUTOREGISTER
  namespace {
    static const bool is_registered = TestManager::RegisterTest(
      "HotStart.Unitary",
      unitary
      );
  }
#endif

  //====================================================================
  int unitary(void)
  {
    int Nc    = CommonParameters::Nc();
    int Nvol  = CommonParameters::Nvol();
    int Ndim  = CommonParameters::Ndim();
    int NinG  = 2 * Nc * Nc;

    Parameters *params_test      = new Parameters_Test_HotStart;

    Parameters *params_all = new Parameters;

    params_all->Register_Parameters("Test_HotStart", params_test);

    ParameterManager_YAML params_manager;
    params_manager.read_params(filename_input, params_all);

    const string str_vlevel      = params_test->get_string("verbose_level");
#ifdef USE_TEST
    const double expected_result = params_test->get_double("expected_result");
#endif

    Bridge::VerboseLevel vl = vout.set_verbose_level(str_vlevel);

    Field_G *u = new Field_G(Nvol, Ndim);
    GaugeConfig::set_hot(u);

    Mat_SU_N link(Nc);
    Mat_SU_N delta(Nc);

    double av = 0;
    for (int site = 0; site < Nvol; ++site) {
      for (int mu = 0; mu < Ndim; ++mu) {
        u->mat(link, site, mu);
        delta.unit();
        delta *= -1;

        // |UU^\dag - I|
        delta.multadd_nd(link, link);
        av += sqrt(delta.norm2());
      }
    }

    double av_all = Communicator::reduce_sum(av);
    int Nlink = CommonParameters::Lvol() * Ndim;

    vout.general(vl, "\n");
    vout.general(vl, "Raondom SU(%d):\n", Nc);
    vout.general(vl, "  number of matrix  = %10d\n", Nlink);
    vout.general(vl, "  ave |UU^dag - I| = %23.16e\n", av_all / Nlink);

    double result = av_all / Nlink;


    // #####  tidy up  #####
    delete params_test;
    delete params_all;

    delete u;


#ifdef USE_TEST
    return Test::verify(expected_result, result);
#else
    return EXIT_SUCCESS;
#endif
  }
} // namespace Test_WilsonLoop
